Class: ActiveSupport::OrderedHash

Inherits:
Hash
  • Object
show all
Defined in:
lib/vitae/ordered_hash.rb

Overview

The order of iteration over hashes in Ruby 1.8 is undefined. For example, you do not know the order in which keys will return keys, or each yield pairs. ActiveSupport::OrderedHash implements a hash that preserves insertion order, as in Ruby 1.9:

oh = ActiveSupport::OrderedHash.new
oh[:a] = 1
oh[:b] = 2
oh.keys # => [:a, :b], this order is guaranteed

ActiveSupport::OrderedHash is namespaced to prevent conflicts with other implementations.

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(*args, &block) ⇒ OrderedHash

In MRI the Hash class is core and written in C. In particular, methods are programmed with explicit C function calls and polymorphism is not honored.

For example, []= is crucial in this implementation to maintain the @keys array but hash.c invokes rb_hash_aset() originally. This prevents method reuse through inheritance and forces us to reimplement stuff.

For instance, we cannot use the inherited #merge! because albeit the algorithm itself would work, our []= is not being called at all by the C code.



46
47
48
49
# File 'lib/vitae/ordered_hash.rb', line 46

def initialize(*args, &block)
  super
  @keys = []
end

Class Method Details

.[](*args) ⇒ Object



51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
# File 'lib/vitae/ordered_hash.rb', line 51

def self.[](*args)
  ordered_hash = new

  if (args.length == 1 && args.first.is_a?(Array))
    args.first.each do |key_value_pair|
      next unless (key_value_pair.is_a?(Array))
      ordered_hash[key_value_pair[0]] = key_value_pair[1]
    end

    return ordered_hash
  end

  unless (args.size % 2 == 0)
    raise ArgumentError.new("odd number of arguments for Hash")
  end

  args.each_with_index do |val, ind|
    next if (ind % 2 != 0)
    ordered_hash[val] = args[ind + 1]
  end

  ordered_hash
end

Instance Method Details

#[]=(key, value) ⇒ Object



81
82
83
84
# File 'lib/vitae/ordered_hash.rb', line 81

def []=(key, value)
  @keys << key unless has_key?(key)
  super
end

#clearObject



140
141
142
143
144
# File 'lib/vitae/ordered_hash.rb', line 140

def clear
  super
  @keys.clear
  self
end

#delete(key) ⇒ Object



86
87
88
89
90
91
92
# File 'lib/vitae/ordered_hash.rb', line 86

def delete(key)
  if has_key? key
    index = @keys.index(key)
    @keys.delete_at index
  end
  super
end

#delete_ifObject



94
95
96
97
98
# File 'lib/vitae/ordered_hash.rb', line 94

def delete_if
  super
  sync_keys!
  self
end

#eachObject Also known as: each_pair



134
135
136
# File 'lib/vitae/ordered_hash.rb', line 134

def each
  @keys.each {|key| yield [key, self[key]]}
end

#each_keyObject



126
127
128
# File 'lib/vitae/ordered_hash.rb', line 126

def each_key
  @keys.each { |key| yield key }
end

#each_valueObject



130
131
132
# File 'lib/vitae/ordered_hash.rb', line 130

def each_value
  @keys.each { |key| yield self[key]}
end

#initialize_copy(other) ⇒ Object



75
76
77
78
79
# File 'lib/vitae/ordered_hash.rb', line 75

def initialize_copy(other)
  super
  # make a deep copy of keys
  @keys = other.keys
end

#inspectObject



178
179
180
# File 'lib/vitae/ordered_hash.rb', line 178

def inspect
  "#<OrderedHash #{super}>"
end

#invertObject



174
175
176
# File 'lib/vitae/ordered_hash.rb', line 174

def invert
  OrderedHash[self.to_a.map!{|key_value_pair| key_value_pair.reverse}]
end

#keysObject



110
111
112
# File 'lib/vitae/ordered_hash.rb', line 110

def keys
  @keys.dup
end

#merge(other_hash, &block) ⇒ Object



163
164
165
# File 'lib/vitae/ordered_hash.rb', line 163

def merge(other_hash, &block)
  dup.merge!(other_hash, &block)
end

#merge!(other_hash) ⇒ Object Also known as: update



152
153
154
155
156
157
158
159
# File 'lib/vitae/ordered_hash.rb', line 152

def merge!(other_hash)
  if block_given?
    other_hash.each { |k, v| self[k] = key?(k) ? yield(k, self[k], v) : v }
  else
    other_hash.each { |k, v| self[k] = v }
  end
  self
end

#reject(&block) ⇒ Object



106
107
108
# File 'lib/vitae/ordered_hash.rb', line 106

def reject(&block)
  dup.reject!(&block)
end

#reject!Object



100
101
102
103
104
# File 'lib/vitae/ordered_hash.rb', line 100

def reject!
  super
  sync_keys!
  self
end

#replace(other) ⇒ Object

When replacing with another hash, the initial order of our keys must come from the other hash -ordered or not.



168
169
170
171
172
# File 'lib/vitae/ordered_hash.rb', line 168

def replace(other)
  super
  @keys = other.keys
  self
end

#shiftObject



146
147
148
149
150
# File 'lib/vitae/ordered_hash.rb', line 146

def shift
  k = @keys.first
  v = delete(k)
  [k, v]
end

#to_aObject



122
123
124
# File 'lib/vitae/ordered_hash.rb', line 122

def to_a
  @keys.map { |key| [ key, self[key] ] }
end

#to_hashObject



118
119
120
# File 'lib/vitae/ordered_hash.rb', line 118

def to_hash
  self
end

#to_yaml(opts = {}) ⇒ Object



23
24
25
26
27
28
29
30
31
# File 'lib/vitae/ordered_hash.rb', line 23

def to_yaml(opts = {})
  YAML.quick_emit(self, opts) do |out|
    out.seq(taguri, to_yaml_style) do |seq|
      each do |k, v|
        seq.add(k => v)
      end
    end
  end
end

#to_yaml_typeObject

:nodoc:



19
20
21
# File 'lib/vitae/ordered_hash.rb', line 19

def to_yaml_type
  "!tag:yaml.org,2002:omap"
end

#valuesObject



114
115
116
# File 'lib/vitae/ordered_hash.rb', line 114

def values
  @keys.collect { |key| self[key] }
end